@@ -35,7 +35,6 @@
 #define HMAC_CTX_free(c) do {					\
 				    HMAC_CTX_cleanup(c);	\
 				    free(c); } while (0)
-#define HMAC_CTX_reset HMAC_CTX_cleanup
 
 static inline HMAC_CTX *HMAC_CTX_new(void)
 {
@@ -56,10 +55,6 @@ void destroy_esp_ciphers(struct esp *esp)
 		HMAC_CTX_free(esp->hmac);
 		esp->hmac = NULL;
 	}
-	if (esp->pkt_hmac) {
-		HMAC_CTX_free(esp->pkt_hmac);
-		esp->pkt_hmac = NULL;
-	}
 }
 
 static int init_esp_ciphers(struct openconnect_info *vpninfo, struct esp *esp,
@@ -82,8 +77,10 @@ static int init_esp_ciphers(struct openconnect_info *vpninfo, struct esp *esp,
 
 	if (decrypt)
 		ret = EVP_DecryptInit_ex(esp->cipher, encalg, NULL, esp->enc_key, NULL);
-	else
-		ret = EVP_EncryptInit_ex(esp->cipher, encalg, NULL, esp->enc_key, NULL);
+	else {
+		ret = RAND_bytes((void *)&esp->iv, sizeof(esp->iv)) &&
+			EVP_EncryptInit_ex(esp->cipher, encalg, NULL, esp->enc_key, esp->iv);
+	}
 
 	if (!ret) {
 		vpn_progress(vpninfo, PRG_ERR,
@@ -94,8 +91,7 @@ static int init_esp_ciphers(struct openconnect_info *vpninfo, struct esp *esp,
 	EVP_CIPHER_CTX_set_padding(esp->cipher, 0);
 
 	esp->hmac = HMAC_CTX_new();
-	esp->pkt_hmac = HMAC_CTX_new();
-	if (!esp->hmac || !esp->pkt_hmac) {
+	if (!esp->hmac) {
 		destroy_esp_ciphers(esp);
 		return -ENOMEM;
 	}
@@ -187,10 +183,9 @@ int decrypt_esp_packet(struct openconnect_info *vpninfo, struct esp *esp, struct
 	unsigned int hmac_len = sizeof(hmac_buf);
 	int crypt_len = pkt->len;
 
-	HMAC_CTX_copy(esp->pkt_hmac, esp->hmac);
-	HMAC_Update(esp->pkt_hmac, (void *)&pkt->esp, sizeof(pkt->esp) + pkt->len);
-	HMAC_Final(esp->pkt_hmac, hmac_buf, &hmac_len);
-	HMAC_CTX_reset(esp->pkt_hmac);
+	HMAC_Init_ex(esp->hmac, NULL, 0, NULL, NULL);
+	HMAC_Update(esp->hmac, (void *)&pkt->esp, sizeof(pkt->esp) + pkt->len);
+	HMAC_Final(esp->hmac, hmac_buf, &hmac_len);
 
 	if (memcmp(hmac_buf, pkt->data + pkt->len, 12)) {
 		vpn_progress(vpninfo, PRG_DEBUG,
@@ -223,19 +218,13 @@ int decrypt_esp_packet(struct openconnect_info *vpninfo, struct esp *esp, struct
 int encrypt_esp_packet(struct openconnect_info *vpninfo, struct pkt *pkt)
 {
 	int i, padlen;
-	const int blksize = 16;
+	int blksize = 16;
 	unsigned int hmac_len = 20;
 	int crypt_len;
 
 	/* This gets much more fun if the IV is variable-length */
 	pkt->esp.spi = vpninfo->esp_out.spi;
 	pkt->esp.seq = htonl(vpninfo->esp_out.seq++);
-	if (!RAND_bytes((void *)&pkt->esp.iv, sizeof(pkt->esp.iv))) {
-		vpn_progress(vpninfo, PRG_ERR,
-			     _("Failed to generate random IV for ESP packet:\n"));
-		openconnect_report_ssl_errors(vpninfo);
-		return -EIO;
-	}
 
 	padlen = blksize - 1 - ((pkt->len + 1) % blksize);
 	for (i=0; i<padlen; i++)
@@ -243,13 +232,7 @@ int encrypt_esp_packet(struct openconnect_info *vpninfo, struct pkt *pkt)
 	pkt->data[pkt->len + padlen] = padlen;
 	pkt->data[pkt->len + padlen + 1] = 0x04; /* Legacy IP */
 
-	if (!EVP_EncryptInit_ex(vpninfo->esp_out.cipher, NULL, NULL, NULL,
-				pkt->esp.iv)) {
-		vpn_progress(vpninfo, PRG_ERR,
-			     _("Failed to set up encryption context for ESP packet:\n"));
-		openconnect_report_ssl_errors(vpninfo);
-		return -EINVAL;
-	}
+	memcpy(pkt->esp.iv, vpninfo->esp_out.iv, 16);
 
 	crypt_len = pkt->len + padlen + 2;
 	if (!EVP_EncryptUpdate(vpninfo->esp_out.cipher, pkt->data, &crypt_len,
@@ -260,10 +243,11 @@ int encrypt_esp_packet(struct openconnect_info *vpninfo, struct pkt *pkt)
 		return -EINVAL;
 	}
 
-	HMAC_CTX_copy(vpninfo->esp_out.pkt_hmac, vpninfo->esp_out.hmac);
-	HMAC_Update(vpninfo->esp_out.pkt_hmac, (void *)&pkt->esp, sizeof(pkt->esp) + crypt_len);
-	HMAC_Final(vpninfo->esp_out.pkt_hmac, pkt->data + crypt_len, &hmac_len);
-	HMAC_CTX_reset(vpninfo->esp_out.pkt_hmac);
+	HMAC_Init_ex(vpninfo->esp_out.hmac, NULL, 0, NULL, NULL);
+	HMAC_Update(vpninfo->esp_out.hmac, (void *)&pkt->esp, sizeof(pkt->esp) + crypt_len);
+	HMAC_Final(vpninfo->esp_out.hmac, pkt->data + crypt_len, &hmac_len);
 
+	EVP_EncryptUpdate(vpninfo->esp_out.cipher, vpninfo->esp_out.iv, &blksize,
+			  pkt->data + crypt_len + hmac_len - blksize, blksize);
 	return sizeof(pkt->esp) + crypt_len + 12;
 }
